package com.yizhi.forum.application.feign.student;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.forum.application.vo.forum.PostsParamsVo;
import com.yizhi.forum.application.vo.forum.PostsSaveVo;
import com.yizhi.forum.application.vo.forum.PostsStudentVo;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@FeignClient(name = "cloud-forum", contextId = "StudentPostsClient")
public interface StudentPostsClient {
    /**
     * @param type       1:最新讨论、2：最热讨论、3：精选热帖
     * @param keyword    分类名称或帖子标题
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */

    @GetMapping("/posts/student/getPageToStudent")
    public Page<PostsStudentVo> getPageToStudent(@RequestParam("type") Integer type,
                                                 @RequestParam(value = "keyword",required = false) String keyword,
                                                 @RequestParam(value = "classifyId",required = false) Long classifyId,
                                                 @RequestParam("pageNo") Integer pageNo,
                                                 @RequestParam("pageSize") Integer pageSize);

    @PostMapping("/posts/student/save")
    public Integer studentSave(@RequestBody PostsSaveVo vo);

    /**
     * 学员查看完整帖子
     *
     * @param vo
     * @return
     */
    @PostMapping("/posts/student/get")
    public PostsSaveVo studentGet(@RequestBody PostsParamsVo vo);


    /**
     * 我的发帖 我的收藏 我的管理 分页数据
     *
     * @param type       1：我的发帖 2：我的收藏 3：我的管理
     * @param status     0：草稿  1：下架  2:上架
     * @param keyword    分类名称或帖子标题
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/posts/student/myPage")
    public Page<PostsStudentVo> myPage(@RequestParam("type") Integer type,
                                       @RequestParam(value = "status",required = false) Integer status,
                                       @RequestParam(value = "keyword",required = false) String keyword,
                                       @RequestParam(value = "classifyId",required = false) Long classifyId,
                                       @RequestParam("pageNo") Integer pageNo,
                                       @RequestParam("pageSize") Integer pageSize);

    /**
     * 学员端修改帖子
     *
     * @param vo
     * @return
     */
    @PostMapping("/posts/student/update")
    public Integer update(@RequestBody PostsSaveVo vo);

    @PostMapping("/posts/student/deleted")
    public Integer deleted(@RequestBody PostsParamsVo vo);

    @PostMapping("/posts/student/topOrHighlight")
    public Integer topOrHighlight(@RequestBody PostsParamsVo vo);

    @GetMapping("/posts/student/getMapByIds")
    public Map<Long, PostsStudentVo> getMapByIds(@RequestParam("postsIds") List<Long> postsIds);

    @GetMapping("/posts/student/getMyNum")
    @ApiOperation(value = "获取我的管理（有回复、举报或帖子未查看）的数量")
    public Integer getMyNum();
}

