package com.yizhi.forum.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.domain.PostsClassify;
import com.yizhi.forum.application.feign.PostsClassifyClient;
import com.yizhi.forum.application.service.IPostsClassifyService;
import com.yizhi.forum.application.vo.domain.PostsClassifyVo;
import com.yizhi.forum.application.vo.forum.PostsClassifySettingVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * <p>
 * 分类接口
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@RestController
@RequestMapping("/postsClassify/manage/")
public class PostsClassifyController implements PostsClassifyClient {

    private Logger logger = LoggerFactory.getLogger(PostsClassifyController.class);


    @Autowired
    private IPostsClassifyService postsClassifyService;


    /**
     * 判断分类名称是否已存在
     *
     * @param name
     * @return
     */
    @Override
    @GetMapping("/exsitsSameName")
    public Boolean exsitsSameName(@RequestParam("name") String name) {
        RequestContext context = ContextHolder.get();
        PostsClassify postsClassify = new PostsClassify();
        postsClassify.setName(name);
        postsClassify.setSiteId(context.getSiteId());
        EntityWrapper<PostsClassify> wrapper = new EntityWrapper(postsClassify);
        if (postsClassifyService.selectCount(wrapper) > 0) {
            return true;
        }
        return false;
    }


    @Override
    @PostMapping("/save")
    public Boolean save(@RequestBody PostsClassifyVo postsClassifyVo) {
        PostsClassifyVo postsClassify = new PostsClassifyVo();

        BeanUtils.copyProperties(postsClassifyVo, postsClassify);
        return postsClassifyService.save(postsClassify);
    }

    @Override
    @PostMapping("/update")
    public Boolean update(@RequestBody PostsClassifyVo postsClassifyVo) {
        RequestContext context = ContextHolder.get();
        PostsClassify postsClassify = new PostsClassify();
        BeanUtils.copyProperties(postsClassifyVo, postsClassify);
        postsClassify.setUpdateById(context.getAccountId());
        postsClassify.setUpdateByName(context.getAccountName());
        postsClassify.setUpdateTime(new Date());

        return postsClassifyService.updateById(postsClassify);
    }

    @Override
    /**
     * type；1-上移  2-下移 必填
     */
    @GetMapping("/move")
    public Integer move(@RequestParam("type") Integer type,
                        @RequestParam("id") Long id) {

        return postsClassifyService.move(type, id);
    }

    @Override
    @GetMapping("/get")
    public PostsClassifyVo get(@RequestParam("id") Long id) {
        PostsClassify postsClassify = this.postsClassifyService.selectById(id);
        if (postsClassify != null) {
            PostsClassifyVo vo = new PostsClassifyVo();
            BeanUtils.copyProperties(postsClassify, vo);
            return vo;
        }
        return null;
    }

    @Override
    @GetMapping("/deleted")
    public Integer deleted(@RequestParam("id") Long id) {
        return postsClassifyService.delete(id);
    }

    /**
     * 更新论坛设置数据包括设置的人员范围
     *
     * @param vo
     * @return
     */
    @Override
    @PostMapping("/updateSetting")
    public Boolean updateSetting(@RequestBody PostsClassifySettingVo vo) {
        return postsClassifyService.updateSetting(vo);
    }

    /**
     * 获取论坛设置数据包括设置的人员范围
     *
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getSetting")
    public PostsClassifySettingVo getSetting(@RequestParam("id") Long id) {
        return postsClassifyService.getSetting(id);
    }

    /**
     * 分页查询
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/getPage")
    public Page<PostsClassifyVo> getPage(@RequestParam(value = "name",required = false) String name,
                                         @RequestParam("pageNo") Integer pageNo,
                                         @RequestParam("pageSize") Integer pageSize) {
        return postsClassifyService.getPage(name, pageNo, pageSize);
    }
}


