package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.domain.PostsClassify;
import com.yizhi.forum.application.domain.PostsClassifyAccount;
import com.yizhi.forum.application.mapper.PostsClassifyAccountMapper;
import com.yizhi.forum.application.service.IPostsClassifyAccountService;
import com.yizhi.forum.application.service.IPostsClassifyService;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.domain.PostsClassifyVo;
import com.yizhi.forum.application.vo.forum.PostsClassifyAccountVo;
import com.yizhi.forum.application.vo.forum.PostsClassifySettingVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsClassifyAccountServiceImpl extends ServiceImpl<PostsClassifyAccountMapper, PostsClassifyAccount> implements IPostsClassifyAccountService {

    private Logger logger = LoggerFactory.getLogger(PostsClassifyAccountServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IPostsService postsService;
    @Autowired
    private IPostsClassifyService postsClassifyService;

    @Override
    public Boolean save(PostsClassifySettingVo vo) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        if (!vo.getIsSave()) {
            //先删除
            PostsClassifyAccount classifyAccount = new PostsClassifyAccount();
            classifyAccount.setDeleted(ForumConstans.DELETED);
            classifyAccount.setUpdateById(context.getAccountId());
            classifyAccount.setUpdateByName(context.getAccountName());
            classifyAccount.setUpdateTime(now);
            EntityWrapper wrapper = new EntityWrapper();
            wrapper.eq(ForumConstans.Field_CLASSIFY_ID, vo.getId());
            this.update(classifyAccount, wrapper);
        }

        if (!CollectionUtils.isEmpty(vo.getPostsClassifyAccountVos())) {
            if (vo.getCommentSetting() == 1 || vo.getSendSetting() == 1) {
                List<PostsClassifyAccount> postsClassifyAccounts = new ArrayList<>(vo.getPostsClassifyAccountVos().size());
                vo.getPostsClassifyAccountVos().forEach(a -> {
                    PostsClassifyAccount classifyAccount = new PostsClassifyAccount();
                    BeanUtils.copyProperties(a, classifyAccount);
                    classifyAccount.setId(idGenerator.generate());
                    classifyAccount.setPostsClassifyId(vo.getId());
                    classifyAccount.setSiteId(context.getSiteId());
                    classifyAccount.setCreateById(context.getAccountId());
                    classifyAccount.setCreateByName(context.getAccountName());
                    classifyAccount.setCreateTime(now);
                    postsClassifyAccounts.add(classifyAccount);
                });
                if (!CollectionUtils.isEmpty(postsClassifyAccounts)) {
                    return this.insertBatch(postsClassifyAccounts);
                }
            }
        }
        return false;
    }

    @Override
    public List<PostsClassifyAccountVo> getListByClassifyId(Long classifyId) {
        PostsClassifyAccount classifyAccount = new PostsClassifyAccount();
        classifyAccount.setSiteId(ContextHolder.get().getSiteId());
        classifyAccount.setPostsClassifyId(classifyId);
        EntityWrapper<PostsClassifyAccount> wrapper = new EntityWrapper(classifyAccount);
        List<PostsClassifyAccount> postsClassifyAccounts = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsClassifyAccounts)) {
            List<PostsClassifyAccountVo> classifyAccountVos = new ArrayList<>(postsClassifyAccounts.size());
            postsClassifyAccounts.forEach(a -> {
                PostsClassifyAccountVo vo = new PostsClassifyAccountVo();
                BeanUtils.copyProperties(a, vo);
                classifyAccountVos.add(vo);
            });
            return classifyAccountVos;
        }
        return new ArrayList<>();
    }

    @Override
    public List<PostsClassifyAccount> getListByAccountId() {
        RequestContext context = ContextHolder.get();
        PostsClassifyAccount classifyAccount = new PostsClassifyAccount();
        classifyAccount.setSiteId(context.getSiteId());
        classifyAccount.setType(ForumConstans.SEND_POSTS_TYPE);
        EntityWrapper<PostsClassifyAccount> wrapper = new EntityWrapper(classifyAccount);
        wrapper.in(ForumConstans.Field_ACCOUNT_ID, context.getRelationIds());
        return this.selectList(wrapper);
    }

    @Override
    public Integer canComment(Long postsId) {
        RequestContext context = ContextHolder.get();
        Posts posts = postsService.selectById(postsId);
        if (posts != null) {
            PostsClassify postsClassify = postsClassifyService.selectById(posts.getPostsClassifyId());
            if (postsClassify != null) {
                if (ForumConstans.SITE_ACCOUNT.equals(postsClassify.getCommentSetting())) {
                    return 1;
                } else {
                    PostsClassifyAccount classifyAccount = new PostsClassifyAccount();
                    classifyAccount.setSiteId(context.getSiteId());
                    classifyAccount.setType(ForumConstans.COMMENT_TYPE);
                    classifyAccount.setPostsClassifyId(posts.getPostsClassifyId());
                    EntityWrapper wrapper = new EntityWrapper(classifyAccount);
                    wrapper.in(ForumConstans.Field_ACCOUNT_ID, context.getRelationIds());
                    return this.selectCount(wrapper);
                }
            }
        } else logger.info("无效id！！");
        return 0;
    }
}
