package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsContent;
import com.yizhi.forum.application.mapper.PostsContentMapper;
import com.yizhi.forum.application.service.IPostsContentService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsContentServiceImpl extends ServiceImpl<PostsContentMapper, PostsContent> implements IPostsContentService {

    @Autowired
    private IdGenerator idGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(Long postsId, String content) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        PostsContent postsContent = new PostsContent();
        postsContent.setId(idGenerator.generate());
        postsContent.setContent(content);
        postsContent.setPostsId(postsId);
        postsContent.setCreateById(context.getAccountId());
        postsContent.setCreateByName(context.getAccountName());
        postsContent.setCreateTime(now);
        return this.insert(postsContent) == true ? 1 : 0;
    }

    @Override
    public String getOneByPostsId(Long postId) {
        PostsContent postsContent = new PostsContent();
        postsContent.setPostsId(postId);
        EntityWrapper wrapper = new EntityWrapper(postsContent);
        PostsContent postsContent1 = this.selectOne(wrapper);
        if (postsContent1 != null) {
            return postsContent1.getContent();
        }
        return "";
    }

    @Override
    public Integer updateByPostsId(Long postsId, String content) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        PostsContent postsContent = new PostsContent();
        postsContent.setContent(content);
        postsContent.setUpdateById(context.getAccountId());
        postsContent.setUpdateByName(context.getAccountName());
        postsContent.setUpdateTime(now);
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq(ForumConstans.FEIlD_POSTS_ID, postsId);
        wrapper.eq("deleted", ForumConstans.NOT_DELETED);

        return this.update(postsContent, wrapper) == true ? 1 : 0;
    }

    @Override
    public Map<Long, String> getMap(List<Long> postsIds) {
        Map<Long, String> map = null;
        PostsContent postsContent = new PostsContent();
        EntityWrapper wrapper = new EntityWrapper(postsContent);
        wrapper.in(ForumConstans.FEIlD_POSTS_ID, postsIds);
        List<PostsContent> list = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(list)) {
            map = new HashMap<>(list.size());
            for (PostsContent a : list) {
                if (!map.containsKey(a.getPostsId())) {
                    map.put(a.getPostsId(), a.getContent());
                }
            }
        }
        return map;
    }
}
