package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsForbiddenAccount;
import com.yizhi.forum.application.mapper.PostsForbiddenAccountMapper;
import com.yizhi.forum.application.service.IPostsForbiddenAccountService;
import com.yizhi.forum.application.vo.domain.PostsForbiddenAccountVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsForbiddenAccountServiceImpl extends ServiceImpl<PostsForbiddenAccountMapper, PostsForbiddenAccount> implements IPostsForbiddenAccountService {
    @Autowired
    private IdGenerator idGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(PostsForbiddenAccountVo postsForbiddenAccount) {

        RequestContext context = ContextHolder.get();
        PostsForbiddenAccount forbiddenAccount = new PostsForbiddenAccount();

        forbiddenAccount.setId(idGenerator.generate());
        forbiddenAccount.setAccountId(postsForbiddenAccount.getAccountId());
        forbiddenAccount.setPostsId(postsForbiddenAccount.getPostsId());
        forbiddenAccount.setSiteId(context.getSiteId());
        forbiddenAccount.setCreateById(context.getAccountId());
        forbiddenAccount.setCreateByName(context.getAccountName());
        forbiddenAccount.setCreateTime(new Date());
        return this.insert(forbiddenAccount) == true ? 1 : 0;
    }

    @Override
    public Page<PostsForbiddenAccountVo> getPage(Long postsId, Integer pageNo, Integer pageSize) {
        RequestContext context = ContextHolder.get();
        Page<PostsForbiddenAccount> page = new Page<>(pageNo, pageSize);
        PostsForbiddenAccount forbiddenAccount = new PostsForbiddenAccount();
        forbiddenAccount.setSiteId(context.getSiteId());
        forbiddenAccount.setPostsId(postsId);
        EntityWrapper wrapper = new EntityWrapper(forbiddenAccount);

        return this.selectPage(page, wrapper);
    }

    @Override
    public Integer delete(PostsForbiddenAccountVo forbiddenAccount) {
        RequestContext context = ContextHolder.get();
        PostsForbiddenAccount forbiddenAccount1 = new PostsForbiddenAccount();
        forbiddenAccount1.setDeleted(ForumConstans.DELETED);
        forbiddenAccount1.setUpdateById(context.getAccountId());
        forbiddenAccount1.setUpdateByName(context.getAccountName());
        forbiddenAccount1.setUpdateTime(new Date());
        if (forbiddenAccount.getId() != null && forbiddenAccount.getId() != 0) {
            forbiddenAccount1.setId(forbiddenAccount.getId());
            return this.updateById(forbiddenAccount1) == true ? 1 : 0;
        }
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq(ForumConstans.Field_ACCOUNT_ID, forbiddenAccount.getAccountId())
                .eq(ForumConstans.FEIlD_POSTS_ID, forbiddenAccount.getPostsId());
        return this.update(forbiddenAccount1, wrapper) == true ? 1 : 0;
    }

    @Override
    public Integer hasForbiddenByPostsId(Long postsId) {
        RequestContext context = ContextHolder.get();
        PostsForbiddenAccount forbiddenAccount = new PostsForbiddenAccount();
        forbiddenAccount.setSiteId(context.getSiteId());
        forbiddenAccount.setPostsId(postsId);
        forbiddenAccount.setAccountId(context.getAccountId());
        EntityWrapper wrapper = new EntityWrapper(forbiddenAccount);
        return this.selectCount(wrapper);
    }

    @Override
    public List<PostsForbiddenAccountVo> getListByPostsId(List<Long> postsIds) {
        if (CollectionUtils.isEmpty(postsIds)) {
            return null;
        }
        PostsForbiddenAccount forbiddenAccount = new PostsForbiddenAccount();
        forbiddenAccount.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper wrapper = new EntityWrapper(forbiddenAccount);
        wrapper.in(ForumConstans.FEIlD_POSTS_ID, postsIds);
        return this.selectList(wrapper);
    }


    @Override
    public Map<Long, Integer> getPersonalDataMap(Boolean isBe) {

        PostsForbiddenAccount forbiddenAccount = new PostsForbiddenAccount();
        EntityWrapper<PostsForbiddenAccount> wrapper = new EntityWrapper(forbiddenAccount);
        List<PostsForbiddenAccount> forbiddenAccounts = this.selectList(wrapper);
        Map<Long, Integer> map = null;
        if (!CollectionUtils.isEmpty(forbiddenAccounts)) {
            map = new HashMap<>(forbiddenAccounts.size());
            for (PostsForbiddenAccount a : forbiddenAccounts) {
                if (isBe) {
                    //查询被禁言记录
                    if (!map.containsKey(a.getAccountId())) {
                        map.put(a.getAccountId(), 1);
                    } else {
                        Integer num = map.get(a.getAccountId());
                        map.put(a.getAccountId(), num + 1);
                    }
                } else {
                    //查询禁言别人的记录
                    if (!map.containsKey(a.getCreateById())) {
                        map.put(a.getCreateById(), 1);
                    } else {
                        Integer num = map.get(a.getCreateById());
                        map.put(a.getCreateById(), num + 1);
                    }
                }
            }
        }
        return map;
    }
}
