package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsManageAccount;
import com.yizhi.forum.application.mapper.PostsManageAccountMapper;
import com.yizhi.forum.application.service.IPostsManageAccountService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.forum.application.vo.forum.PostsManageAccountParamVo;
import com.yizhi.forum.application.vo.domain.PostsManageAccountVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsManageAccountServiceImpl extends ServiceImpl<PostsManageAccountMapper, PostsManageAccount> implements IPostsManageAccountService {

    @Autowired
    private IdGenerator idGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean save(PostsManageAccountParamVo vo) {
        RequestContext context = ContextHolder.get();
        Date date = new Date();


        if (!vo.getIsSave()) {
            //先删除
            PostsManageAccount manageAccount = new PostsManageAccount();
            manageAccount.setDeleted(ForumConstans.DELETED);
            manageAccount.setUpdateById(context.getAccountId());
            manageAccount.setUpdateByName(context.getAccountName());
            manageAccount.setUpdateTime(date);
            EntityWrapper wrapper = new EntityWrapper();
            wrapper.eq(ForumConstans.Field_CLASSIFY_ID, vo.getPostsClassifyId());
            this.update(manageAccount, wrapper);
        }

        List<PostsManageAccountVo> manageAccounts = vo.getPostsManageAccounts();
        List<PostsManageAccount> manageAccounts1 = new ArrayList<>();
        if (!CollectionUtils.isEmpty(manageAccounts)) {
            for (PostsManageAccountVo a : manageAccounts) {
                a.setId(idGenerator.generate());
                a.setPostsClassifyId(vo.getPostsClassifyId());
                a.setSiteId(context.getSiteId());
                a.setCreateById(context.getAccountId());
                a.setCreateByName(context.getAccountName());
                a.setCreateTime(date);
                PostsManageAccount postsManageAccount=new PostsManageAccount();
                BeanUtils.copyProperties(a,postsManageAccount);
                manageAccounts1.add(postsManageAccount);
            }
            return this.insertBatch(manageAccounts1);
        }
        return true;
    }

    @Override
    public List<com.yizhi.forum.application.vo.forum.PostsManageAccountVo> getManager(Long postsClassifyId, Long siteId) {

        PostsManageAccount manageAccount = new PostsManageAccount();
        manageAccount.setSiteId(siteId);
        manageAccount.setPostsClassifyId(postsClassifyId);
        EntityWrapper<PostsManageAccount> wrapper = new EntityWrapper(manageAccount);
        List<PostsManageAccount> manageAccounts = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(manageAccounts)) {
            List<com.yizhi.forum.application.vo.forum.PostsManageAccountVo> manageAccountVos = new ArrayList<>(manageAccounts.size());
            manageAccounts.forEach(a -> {
                com.yizhi.forum.application.vo.forum.PostsManageAccountVo vo = new com.yizhi.forum.application.vo.forum.PostsManageAccountVo();
                BeanUtils.copyProperties(a, vo);
                manageAccountVos.add(vo);
            });
            return manageAccountVos;
        }
        return new ArrayList<>();

    }

    @Override
    public List<Long> getClassifyIds(Long accountId, Long siteId) {
        PostsManageAccount manageAccount = new PostsManageAccount();
        manageAccount.setSiteId(siteId);
        manageAccount.setAccountId(accountId);
        EntityWrapper<PostsManageAccount> wrapper = new EntityWrapper(manageAccount);
        List<PostsManageAccount> manageAccounts = this.selectList(wrapper);
        return manageAccounts.stream().map(a -> a.getPostsClassifyId()).collect(Collectors.toList());
    }

}
