package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.domain.PostsReadRecord;
import com.yizhi.forum.application.mapper.PostsReadRecordsMapper;
import com.yizhi.forum.application.service.IPostsManageAccountService;
import com.yizhi.forum.application.service.IPostsReadRecordsService;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.domain.PostsVo;
import com.yizhi.forum.application.vo.forum.PostsManageAccountVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsReadRecordsServiceImpl extends ServiceImpl<PostsReadRecordsMapper, PostsReadRecord> implements IPostsReadRecordsService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private PostsReadRecordsMapper postsReadRecordsMapper;
    @Autowired
    private IPostsService postsService;
    @Autowired
    private IPostsManageAccountService postsManageAccountService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertRecord(Integer type, Long relationId, Integer terminal) {
        RequestContext context = ContextHolder.get();
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                PostsReadRecord readRecord = new PostsReadRecord();
                readRecord.setId(idGenerator.generate());
                readRecord.setAccountId(context.getAccountId());
                readRecord.setTerminalType(terminal);
                readRecord.setPostsId(relationId);
                readRecord.setCreateById(context.getAccountId());
                readRecord.setCreateByName(context.getAccountName());
                readRecord.setCreateTime(new Date());
                readRecord.setSiteId(context.getSiteId());
                if (postsReadRecordsMapper.insert(readRecord) > 0) {
                    //更新帖子的阅读数字段,以及是否被管理员管理字段
                    Posts posts = postsService.selectById(relationId);
                    if (posts != null) {
                        PostsReadRecord tem = new PostsReadRecord();
                        tem.setSiteId(context.getSiteId());
                        tem.setPostsId(relationId);
                        EntityWrapper wrapper = new EntityWrapper(tem);
                        wrapper.ne(ForumConstans.FEIlD_TERMINAL_TYPE, ForumConstans.TERMINAL_MANAGE);
                        PostsVo vo = new PostsVo();
                        vo.setId(relationId);
                        vo.setReadNum(postsReadRecordsMapper.selectCount(wrapper));

                        //如果是已被管理了，则不需要查询啦
                        if (!posts.getHasManage().equals(1)) {
                            //获取代管员名单
                            List<PostsManageAccountVo> manager = postsManageAccountService.getManager(posts.getPostsClassifyId(),context.getSiteId());
                            Set<Long> managerIds = null;
                            if (!CollectionUtils.isEmpty(manager)) {
                                managerIds = manager.stream().map(a -> a.getAccountId()).collect(Collectors.toSet());
                            }
                            EntityWrapper wrapper1 = new EntityWrapper(tem);
                            wrapper1.eq(ForumConstans.FEIlD_TERMINAL_TYPE, ForumConstans.TERMINAL_MANAGE);
                            if (!CollectionUtils.isEmpty(manager)) {
                                wrapper1.orNew().ne(ForumConstans.FEIlD_TERMINAL_TYPE, ForumConstans.TERMINAL_MANAGE)
                                        .eq(ForumConstans.FEIlD_POSTS_ID,posts.getId())
                                        .in(ForumConstans.Field_ACCOUNT_ID, managerIds);
                            }
                            vo.setHasManage(postsReadRecordsMapper.selectCount(wrapper1) > 0 ? 1 : 0);
                        }
                        postsService.updateInfo(vo);
                    }
                }
            }
        });
        return 1;
    }

    @Override
    public Map<Long, Integer> getPersonalDataMap() {

        Map<Long, Integer> map = null;
        PostsReadRecord readRecord = new PostsReadRecord();
        List<PostsReadRecord> postsReadRecords = this.selectList(new EntityWrapper<>(readRecord));
        if (!CollectionUtils.isEmpty(postsReadRecords)) {
            map = new HashMap<>(postsReadRecords.size());
            for (PostsReadRecord a : postsReadRecords) {
                if (!map.containsKey(a.getAccountId())) {
                    map.put(a.getAccountId(), 1);
                } else {
                    Integer num = map.get(a.getAccountId());
                    map.put(a.getAccountId(), num + 1);
                }
            }
        }
        return map;
    }

    @Override
    public Map<Long, Integer> getTaskMap(Integer type, List<Long> taskIds) {

        Map<Long, Integer> map = null;
        if (CollectionUtils.isEmpty(taskIds)) {
            return map;
        }
        PostsReadRecord readRecord = new PostsReadRecord();
       readRecord.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper<PostsReadRecord> wrapper = new EntityWrapper<>(readRecord);
        wrapper.in(ForumConstans.FEIlD_POSTS_ID, taskIds);
        List<PostsReadRecord> postsReadRecords = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsReadRecords)) {
            map = new HashMap<>(postsReadRecords.size());
            for (PostsReadRecord a : postsReadRecords) {
                if (!map.containsKey(a.getAccountId())) {
                    map.put(a.getPostsId(), 1);
                } else {
                    Integer num = map.get(a.getAccountId());
                    map.put(a.getPostsId(), num + 1);
                }
            }
        }
        return map;
    }
}