package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.domain.PostsClassify;
import com.yizhi.forum.application.mapper.PostsMapper;
import com.yizhi.forum.application.service.*;
import com.yizhi.forum.application.util.ForumUtil;
import com.yizhi.forum.application.vo.domain.PostsClassifyVo;
import com.yizhi.forum.application.vo.domain.PostsVo;
import com.yizhi.forum.application.vo.forum.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsServiceImpl extends ServiceImpl<PostsMapper, Posts> implements IPostsService {

    private Logger logger = LoggerFactory.getLogger(PostsServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IPostsService postsService;
    @Autowired
    private IPostsContentService postsContentService;
    @Autowired
    private IPostsAppendixService postsAppendixService;
    @Autowired
    private IPostsVisibleRangeService postsVisibleRangeService;
    @Autowired
    private IPostsRelationService postsRelationService;
    @Autowired
    private IPostsClassifyService postsClassifyService;
    @Autowired
    private IPostsFavoriteService postsFavoriteService;
    @Autowired
    private IPostsManageAccountService manageAccountService;
    @Autowired
    private IPostsReadRecordsService postsReadRecordsService;
    @Autowired
    private PostsMapper postsMapper;
    @Autowired
    private IPostsForbiddenAccountService postsForbiddenAccountService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(PostsSaveVo vo) {
        RequestContext context = ContextHolder.get();
        Long postsId = 0L;
        if (vo.getId() == null) {
            postsId = idGenerator.generate();
        } else {
            postsId = vo.getId();
        }

        Date now = new Date();

        Posts posts = new Posts();
        BeanUtils.copyProperties(vo, posts);
        posts.setId(postsId);
        posts.setAuthor(context.getAccountId());
        posts.setSiteId(context.getSiteId());
        posts.setCompanyId(context.getCompanyId());
        posts.setOrgId(context.getOrgId());
        posts.setCreateById(context.getAccountId());
        posts.setCreateByName(context.getAccountName());
        posts.setCreateTime(now);
        //初始值为创建时间
        posts.setTopTime(now);
        posts.setCommentTime(now);
        posts.setHighlightTime(now);
        posts.setSortTime(now);
        if (ForumConstans.UP.equals(vo.getType())) {
            posts.setStatus(ForumConstans.UP);
        }
        this.insert(posts);

        //保存帖子内容
        postsContentService.save(postsId, vo.getContent());

        //保存附件
        if (!CollectionUtils.isEmpty(vo.getPostsAppendixVos())) {
            postsAppendixService.save(postsId, vo.getPostsAppendixVos());
        }

        //只有管理端才能保存以下数据
        if (ForumConstans.TERMINAL_MANAGE.equals(vo.getTerminalType())) {
            //保存关联活动
            if (!CollectionUtils.isEmpty(vo.getPostsRelationVos())) {
                postsRelationService.save(postsId, vo.getPostsRelationVos());
            }

            //保存指定用户
            if (!CollectionUtils.isEmpty(vo.getPostsVisibleRangeVos()) && ForumConstans.VISIBLE_RANGE.equals(vo.getVisibleRange())) {
                postsVisibleRangeService.save(postsId, vo.getPostsVisibleRangeVos());
            }
        }

        return 1;
    }

    @Override
    public PostsSaveVo get(PostsParamsVo paramsVo) {
        PostsSaveVo vo = new PostsSaveVo();
        //获取帖子基本信息
        Posts posts = this.selectById(paramsVo.getId());
        BeanUtils.copyProperties(posts, vo);
        vo.setCreateTimeString(ForumUtil.getCreatTimeString(posts.getUpdateTime()));
        //获取帖子内容
        String content = postsContentService.getOneByPostsId(paramsVo.getId());
        vo.setContent(content);
        //获取该帖子的阅读记录
        vo.setReadNum(posts.getReadNum());
        //获取分类名称
        PostsClassify postsClassify = postsClassifyService.selectById(posts.getPostsClassifyId());
        if (postsClassify != null) {
            vo.setPostClassifyName(postsClassify.getName());
        }
        //判断当前用户是否收藏了
        vo.setHasFavorite(postsFavoriteService.hasFavoriteByPostsId(paramsVo.getId(), true) > 0 ? 1 : 0);

        //获取帖子的附件
        vo.setPostsAppendixVos(postsAppendixService.getListByPostsId(paramsVo.getId()));

        //获取帖子的关联活动
        vo.setPostsRelationVos(postsRelationService.getListByPostsId(paramsVo.getId()));

        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer update(PostsSaveVo vo) {
        RequestContext context = ContextHolder.get();

        Long postsId = vo.getId();
        Posts posts = new Posts();
        BeanUtils.copyProperties(vo, posts);
        posts.setStatus(vo.getType());
        //修改字段赋值
        setUpdateInfo(posts, context);
        this.updateById(posts);

        //保存帖子内容
        postsContentService.updateByPostsId(postsId, vo.getContent());

        //保存附件
        postsAppendixService.deletedByPostsId(postsId);
        postsAppendixService.save(postsId, vo.getPostsAppendixVos());

        //只有管理端才能保存以下数据
        if (ForumConstans.TERMINAL_MANAGE.equals(vo.getTerminalType())) {
            //保存关联活动
            postsRelationService.deletedByPostsId(postsId);
            postsRelationService.save(postsId, vo.getPostsRelationVos());

            //保存指定用户
            postsVisibleRangeService.deletedByPostsId(postsId);
            postsVisibleRangeService.save(postsId, vo.getPostsVisibleRangeVos());
        }

        return 1;
    }

    /**
     * 给修改字段赋值
     *
     * @param posts
     * @param context
     */
    public void setUpdateInfo(Posts posts, RequestContext context) {
        Date now = new Date();
        posts.setUpdateById(context.getAccountId());
        posts.setUpdateByName(context.getAccountName());
        posts.setUpdateTime(now);
    }

    @Override
    public Integer upOrDown(PostsParamsVo vo) {
        Posts posts = new Posts();
        posts.setId(vo.getId());
        if (ForumConstans.UP.equals(vo.getType())) {
            posts.setStatus(ForumConstans.UP);
            posts.setUpdateTime(new Date());

        } else if (ForumConstans.DOWN.equals(vo.getType())) {
            posts.setStatus(ForumConstans.DOWN);
        }
        return this.updateById(posts) == true ? 1 : 0;
    }

    @Override
    public Integer topOrHighlight(PostsParamsVo vo) {
        Date date = new Date();

        Posts tem = this.selectById(vo.getId());
        if (tem == null) {
            logger.info("置顶或加精时，id无效！");
            return 0;
        }
        Posts posts = new Posts();
        posts.setId(vo.getId());
        if (ForumConstans.HIGH_LIGHT.equals(vo.getType())) {
            //是否加精
            if (ForumConstans.IS_HIGH_LIGHT.equals(vo.getIsHighlight())) {
                posts.setHighlightTime(date);
                posts.setIsHighlight(ForumConstans.IS_HIGH_LIGHT);
            } else {
                posts.setIsHighlight(ForumConstans.CANCEL_HIGH_LIGHT);
            }
        } else if (ForumConstans.TOP.equals(vo.getType())) {
            //是否置顶
            if (ForumConstans.IS_TOP.equals(vo.getIsTop())) {
                posts.setIsTop(ForumConstans.IS_TOP);
                posts.setSortTime(ForumConstans.MAX_TOP_DATE);
                posts.setTopTime(date);
            } else {
                posts.setIsTop(ForumConstans.CANCEL_TOP);
                posts.setSortTime(tem.getCommentTime());
            }
        }
        return this.updateById(posts) == true ? 1 : 0;
    }

    @Override
    public Page<PostsManageVo> getPage(PostsParamsVo vo) {
        Page page = new Page<>(vo.getPageNo(), vo.getPageSize());
        RequestContext context = ContextHolder.get();

        Posts posts = new Posts();
        posts.setPostsClassifyId(vo.getPostsClassifyId());
        posts.setSiteId(context.getSiteId());
        EntityWrapper wrapper = new EntityWrapper(posts);
        if (vo.getStatus() != null) {
            switch (vo.getStatus()) {
                case 0:
                    //草稿
                    posts.setStatus(ForumConstans.DRAFT);
                    break;
                case 1:
                    //下架
                    posts.setStatus(ForumConstans.DOWN);
                    break;
                case 2:
                    //上架
                    posts.setStatus(ForumConstans.UP);
                    break;
            }
        }
        if (vo.getType() != null) {
            switch (vo.getType()) {
                case 5:
                    //帖子待管理
                    posts.setHasManage(ForumConstans.POSTS_NOT_MANAGE);
                    break;
                case 6:
                    //举报待管理
                    posts.setHasNewInform(ForumConstans.INFORM_NOT_MANAGE);
                    break;
            }
            //评论待管理
            if (vo.getType().equals(7)) {
                wrapper.gt(ForumConstans.Field_NEW_COMMENT_NUM, ForumConstans.ZERO);
            }
        }
        if (vo.getIsHighlight() != null) {
            posts.setIsHighlight(vo.getIsHighlight() > 0 ? 1 : 0);
        }

        if (!CollectionUtils.isEmpty(vo.getAuthors())) {
            wrapper.in(ForumConstans.Field_AUTHOR, vo.getAuthors());
        }
        if (!StringUtils.isEmpty(vo.getTitle())) {
            wrapper.like(ForumConstans.FEIlD_TITLE, vo.getTitle());
        }
        wrapper.orderBy(ForumConstans.FEIlD_UPDATE_TIME, false);
        page = this.selectPage(page, wrapper);
        List<Posts> postsList = page.getRecords();
        if (!CollectionUtils.isEmpty(postsList)) {
            List<PostsManageVo> postsManageVos = new ArrayList<>(postsList.size());
            postsList.forEach(a -> {
                PostsManageVo manageVo = new PostsManageVo();
                BeanUtils.copyProperties(a, manageVo);
                if (ForumConstans.DRAFT.equals(a.getStatus())) {
                    manageVo.setStatus("草稿");
                } else if (ForumConstans.DOWN.equals(a.getStatus())) {
                    manageVo.setStatus("已下架");
                } else {
                    manageVo.setStatus("已上架");
                }
                postsManageVos.add(manageVo);
            });
            page.setRecords(postsManageVos);
        }
        return page;
    }

    @Override
    public Integer updateInfo(PostsVo vo) {
        Posts posts = new Posts();
        BeanUtils.copyProperties(vo, posts);
        return this.updateById(posts) == true ? 1 : 0;
    }

    @Override
    public Page<PostsStudentVo> getPageToStudent(Integer type, String keyword, Long classifyId, Integer pageNo, Integer pageSize) {
        Page page = new Page<>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        Posts posts = new Posts();
        posts.setSiteId(context.getSiteId());
        posts.setStatus(ForumConstans.UP);

        EntityWrapper wrapper = new EntityWrapper(posts);
        if (classifyId != null) {
            wrapper.eq(ForumConstans.Field_CLASSIFY_ID, classifyId);
        }
        //构建keyword参数的wrapper
        wrapper = buildWrapperByKeyword(keyword, wrapper);

        switch (type) {
            //最近讨论
            case 1:
                wrapper.orderBy(ForumConstans.Field_SORT_TIME, false)
                        .orderBy(ForumConstans.FEIlD_TOP_TIME, false);
                break;
            //火热帖子
            case 2:
                wrapper.orderBy(ForumConstans.Field_READ_NUM, false);
                break;
            //加精热帖
            case 3:
                wrapper.eq(ForumConstans.FEIlD_IS_HIGHLIGHT, ForumConstans.IS_HIGH_LIGHT)
                        .orderBy(ForumConstans.FEIlD_HIGHLIGHT_TIME, false);
                break;
            default:
                wrapper.orderBy(ForumConstans.FEIlD_CREATE_TIME, false);

        }
        //对可见范围做限制
        List<Long> postsIds = postsVisibleRangeService.getPostsIds(context);
        wrapper.andNew().eq(ForumConstans.FEIlD_VISIBLE_RANGE, ForumConstans.SITE_ACCOUNT);
        if (!CollectionUtils.isEmpty(postsIds)) {
            wrapper.or().in(ForumConstans.Field_ID, postsIds);
        }

        page = this.selectPage(page, wrapper);
        List<Posts> postsList = page.getRecords();
        page.setRecords(buildVo(postsList, context, type));

        return page;
    }

    /**
     * 构建keyword参数的wrapper
     *
     * @param keyword
     * @param wrapper
     * @return
     */
    public EntityWrapper buildWrapperByKeyword(String keyword, EntityWrapper wrapper) {
        if (!StringUtils.isEmpty(keyword)) {
            Page<PostsClassifyVo> classifyVoPage = postsClassifyService.getPageToStudent(keyword, false, 1, Integer.MAX_VALUE);
            List<PostsClassifyVo> classifyVos = classifyVoPage.getRecords();
            Set<Long> classifyIds = null;
            if (!CollectionUtils.isEmpty(classifyVos)) {
                classifyIds = new HashSet<>(classifyVos.size());
                for (PostsClassifyVo a : classifyVos) {
                    classifyIds.add(a.getId());
                }
            }
            wrapper.andNew().like(ForumConstans.FEIlD_TITLE, keyword);
            if (!CollectionUtils.isEmpty(classifyIds)) {
                wrapper.or().in(ForumConstans.Field_CLASSIFY_ID, classifyIds);
            }
        }
        return wrapper;
    }

    /**
     * 构建vo
     *
     * @param postsList
     * @param context
     * @param type      1- 最近讨论   2-火热帖子    3-加精热帖    4-我的发帖  5：我的收藏    6：我的管理
     * @return
     */
    public List<PostsStudentVo> buildVo(List<Posts> postsList, RequestContext context, Integer type) {
        if (!CollectionUtils.isEmpty(postsList)) {
            List<PostsStudentVo> postsStudentVos = new ArrayList<>(postsList.size());
            Set postsIds = new HashSet();
            Set postsClassifyIds = new HashSet();
            for (Posts a : postsList) {
                postsIds.add(a.getId());
                postsClassifyIds.add(a.getPostsClassifyId());
            }
            //获取分类
            Map<Long, String> classifyNameMap = postsClassifyService.getClassifyNameMap(new ArrayList<>(postsClassifyIds));

            //获取帖子内容
            Map<Long, String> postsContentMap = postsContentService.getMap(new ArrayList<>(postsIds));

            for (Posts a : postsList) {
                PostsStudentVo vo = new PostsStudentVo();
                BeanUtils.copyProperties(a, vo);
                vo.setContent(postsContentService.getOneByPostsId(a.getId()));
                vo.setCreateTimeString(ForumUtil.getCreatTimeString(a.getUpdateTime()));
                if (classifyNameMap != null) {
                    vo.setPostClassifyName(classifyNameMap.get(a.getPostsClassifyId()));
                }
                if (postsContentMap != null) {
                    vo.setContent(postsContentMap.get(a.getId()));
                }
                postsStudentVos.add(vo);
            }
            return postsStudentVos;
        }
        return new ArrayList<>();
    }


    @Override
    public Page<PostsStudentVo> myPage(Integer type, Integer status, String keyword, Long classifyId, Integer pageNo, Integer pageSize) {
        Page page = new Page<>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        Posts posts = new Posts();
        posts.setSiteId(context.getSiteId());

        EntityWrapper wrapper = new EntityWrapper(posts);
        if (classifyId != null) {
            wrapper.eq(ForumConstans.Field_CLASSIFY_ID, classifyId);
        }

        switch (type) {
            //1：我的发帖 2：我的收藏 3：我的管理
            case 1:
                posts.setAuthor(context.getAccountId());
                if (status != null) {
                    if (status != 1) {
                        wrapper.eq(ForumConstans.FEIlD_STATUS, ForumConstans.UP);
                    } else {
                        wrapper.eq(ForumConstans.FEIlD_STATUS, ForumConstans.DRAFT);
                    }
                }
                type = 4;
                wrapper.orderBy(ForumConstans.FEIlD_UPDATE_TIME, false);
                break;
            case 2:
                posts.setStatus(ForumConstans.UP);
                List<Long> postsIds = postsFavoriteService.getPostsIds(context.getAccountId(), context.getSiteId());
                if (CollectionUtils.isEmpty(postsIds)) {
                    return page;
                }
                wrapper.in(ForumConstans.Field_ID, postsIds);
                wrapper.orderBy(ForumConstans.FEIlD_FAVORITE_TIME, false);
                type = 5;
                break;
            case 3:
                posts.setStatus(ForumConstans.UP);
                List<Long> classifyIds = manageAccountService.getClassifyIds(context.getAccountId(), context.getSiteId());
                if (CollectionUtils.isEmpty(classifyIds)) {
                    return page;
                }
                wrapper.in(ForumConstans.Field_CLASSIFY_ID, classifyIds);
                wrapper.orderBy(ForumConstans.FEIlD_MANAGE_SORT, false);
                type = 6;
                break;
        }
        //构建keyword参数的wrapper
        wrapper = buildWrapperByKeyword(keyword, wrapper);

        page = this.selectPage(page, wrapper);
        List<Posts> postsList = page.getRecords();
        page.setRecords(buildVo(postsList, context, type));

        return page;
    }

    @Override
    public List<Posts> getList(PostsParamsVo vo) {

        Posts posts = new Posts();
        posts.setDeleted(vo.getIncludeDeleted() == false ? ForumConstans.NOT_DELETED : null);
        posts.setAuthor(vo.getAuthor());
        EntityWrapper wrapper = new EntityWrapper(posts);
        wrapper.ne(ForumConstans.FEIlD_STATUS, ForumConstans.DRAFT);
        wrapper.gt(ForumConstans.Field_COMMENT_NUM, ForumConstans.ZERO);
        if (!StringUtils.isEmpty(vo.getTitle())) {
            wrapper.like(ForumConstans.FEIlD_TITLE, vo.getTitle());
        }
        return this.selectList(wrapper);
    }

    @Override
    public Map<Long, PostsStudentVo> getMap(PostsParamsVo vo) {
        Map<Long, PostsStudentVo> map = null;
        Posts posts = new Posts();
        posts.setAuthor(vo.getAuthor());
        EntityWrapper wrapper = new EntityWrapper(posts);
        if (!vo.getIncludeDeleted()) {
            wrapper.eq(ForumConstans.FEIlD_STATUS, ForumConstans.UP);
        }
        if (vo.getTitle() != null) {
            wrapper.like(ForumConstans.FEIlD_TITLE, vo.getTitle());
        }
        List<Posts> postsList = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsList)) {
            List<Long> postsIds = postsList.stream().map(a -> a.getId()).collect(Collectors.toList());
            Map<Long, String> contentMap = postsContentService.getMap(postsIds);
            map = new HashMap<>(postsList.size());
            for (Posts a : postsList) {
                PostsStudentVo studentVo = new PostsStudentVo();
                BeanUtils.copyProperties(a, studentVo);
                if (contentMap != null) {
                    studentVo.setContent(contentMap.get(a.getId()));
                }
                if (!map.containsKey(studentVo.getId())) {
                    map.put(studentVo.getId(), studentVo);
                }
            }
        }
        return map;
    }

    @Override
    public Map<Long, PostsStudentVo> getMapByIds(List<Long> postsIds) {
        if (CollectionUtils.isEmpty(postsIds)) {
            return null;
        }
        Map<Long, PostsStudentVo> map = this.baseMapper.getMapByIds(postsIds);
        return map;
    }

    @Override
    public Map<Long, StatisticsForumAccountVo> getPersonalDataMap() {
        Map<Long, StatisticsForumAccountVo> dataMap = postsMapper.getPersonalDataMap();
        Map<Long, StatisticsForumAccountVo> dataMap1 = new HashMap<>();
        if (dataMap != null) {
            //获取用户对应的被禁言数
            Map<Long, Integer> beForbiddenAccountMap = postsForbiddenAccountService.getPersonalDataMap(true);
            //获取阅读数
            Map<Long, Integer> readRecordsMap = postsReadRecordsService.getPersonalDataMap();

            for (Long accountId : dataMap.keySet()) {
                StatisticsForumAccountVo vos = dataMap.get(accountId);
                StatisticsForumAccountVo vo=new StatisticsForumAccountVo();
                BeanUtils.copyProperties(vos,vo);
                Integer beForbiddenNum = 0;
                Integer readNum = 0;
                if (beForbiddenAccountMap != null) {
                    beForbiddenNum = beForbiddenAccountMap.get(accountId);
                }
                if (readRecordsMap != null) {
                    readNum = readRecordsMap.get(accountId);
                }
                if (vo != null) {
                    vo.setBeForbiddenNum(beForbiddenNum);
                    vo.setReadNum(readNum);
                }
                dataMap1.put(accountId,vo);
            }
        }
        return dataMap1;
    }

    @Override
    public Posts getInfo(Long postsId) {
        Posts posts = this.selectById(postsId);
        if (posts != null) {
            PostsClassify postsClassify = postsClassifyService.selectById(posts.getPostsClassifyId());
            if (postsClassify != null) {
                posts.setSendPoint(postsClassify.getSendPoint());
                posts.setCommentPoint(postsClassify.getCommentPoint());
            }
        }
        return posts;
    }

    @Override
    public Integer getMyNum() {
        RequestContext context = ContextHolder.get();

        Integer myManageNum = 0;
        List<Long> classifyIds = manageAccountService.getClassifyIds(context.getAccountId(), context.getSiteId());

        if (!CollectionUtils.isEmpty(classifyIds)) {
            Posts posts = new Posts();
            posts.setSiteId(context.getSiteId());
            posts.setStatus(ForumConstans.UP);
            EntityWrapper wrapper = new EntityWrapper(posts);
            wrapper.in(ForumConstans.Field_CLASSIFY_ID, classifyIds);
            List<Posts> postsList = this.selectList(wrapper);
            if (!CollectionUtils.isEmpty(postsList)) {
                for (Posts a : postsList) {
                    if (a.getHasManage() == 0 || a.getNewCommentNum() > 0 || a.getHasNewInform() > 0) {
                        myManageNum++;
                    }
                }
            }
        }
        return myManageNum;
    }

    @Override
    public Map<Long, String> getNames(List<Long> postsIds) {
        if (!CollectionUtils.isEmpty(postsIds)) {
            EntityWrapper wrapper = new EntityWrapper();
            wrapper.in("id", postsIds);
            List<Posts> postsList = this.selectList(wrapper);
            if (!CollectionUtils.isEmpty(postsList)) {
                return postsList.stream().collect(Collectors.toMap(Posts::getId, Posts::getTitle));
            }
        }
        return null;
    }
}
